<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 14/01/2016
 * Time: 13:44
 */

namespace Folha\Logic\Cadastros\Evento;


use Folha\Entities\FpaEstrutevento;
use Folha\Entities\FpaOcupaevento;
use Folha\Entities\FpaVincevento;
use Folha\Entities\GpeOcupacaom;
use Folha\Entities\GpeVinculom;
use Geral\Entities\Estruturam;
use HCM\Util\DateUtil;
use Geral\Util\NovoCodigo;
use HCM\Util\FPA;
use HCM\Util\Messages;
use Setforms2\Shared\Registry;

class EventoRelacionado {

    const ESTRUTURA = 'FPA_ESTRUTEVENTO';
    const OCUPACAO = 'FPA_OCUPAEVENTO';
    const VINCULO = 'FPA_VINCEVENTO';

    private $nrorg;
    private $table;
    private $dtinivigencia;
    private $dtfimvigencia;
    private $idtipomovimento;
    private $nreventom;
    private $vrpisoevento;
    private $vrtetoevento;
    private $nrvalreferencia;
    private $vrpercent;
    private $vrmultiplic;
    private $vrdivide;
    private $vreventofixo;
    private $idrefgerar;
    private $vrreffixo;
    private $idretreferencia;
    private $periodicidademes;
    private $nrestruturam;
    private $idgeraafastado;

    public function __construct($nrorg, $table, $dtinivigencia, $dtfimvigencia, $idtipomovimento, $nreventom,
                                $vrpisoevento=0, $vrtetoevento=9999999999, $nrvalreferencia=null, $vrpercent=0, $vrmultiplic=1, $vrdivide=1, $vreventofixo=0,
                                $idrefgerar='NENHUM', $vrreffixo=0, $idretreferencia='N', $periodicidademes='SSSSSSSSSSSS', $nrestruturam, $idgeraafastado = null) {
        $this->nrorg = $nrorg;
        $this->table = $table;
        $this->dtinivigencia = $dtinivigencia;
        $this->dtfimvigencia = $dtfimvigencia;
        $this->idtipomovimento = $idtipomovimento;
        $this->nreventom = $nreventom;
        $this->vrpisoevento = $vrpisoevento;
        $this->vrtetoevento = $vrtetoevento;
        $this->nrvalreferencia = $nrvalreferencia;
        $this->vrpercent = $vrpercent;
        $this->vrmultiplic = $vrmultiplic;
        $this->vrdivide = $vrdivide;
        $this->vreventofixo = $vreventofixo;
        $this->idrefgerar = $idrefgerar;
        $this->vrreffixo = $vrreffixo;
        $this->idretreferencia = $idretreferencia;
        $this->periodicidademes = $periodicidademes;
        $this->nrestruturam = $nrestruturam;
        $this->idgeraafastado = $idgeraafastado;
    }

    public function cadastrarEventoRelacionado($itens)
    {
        try {
            Registry::getEntityManager()->beginTransaction();

            foreach ($itens as $item) {
                if (is_array($this->nreventom)) {
                    foreach ($this->nreventom as $nreventom) {
                        $this->insereNovoEventoRelacionado($item, $nreventom);
                    }
                }
                else {
                    $this->insereNovoEventoRelacionado($item, $this->nreventom);
                }
            }

            Registry::getEntityManager()->flush();

            Registry::getEntityManager()->commit();
        } catch (\Exception $e) {
            Registry::getEntityManager()->rollback();
            throw $e;
        }
    }

    /**
     * @param $nrestruturam
     */
    private function validaEstruturaEvento($nrestruturam, $nreventom)
    {
        try {
            /** @var Estruturam $estrutura */
            $estrutura = Estruturam::getObjectByWhere('NRORG = ? AND NRESTRUTURAM = ?', array($this->nrorg, $nrestruturam));

            if ($estrutura) {
                $where =
                    "NRESTRUTURAM = :P_NRESTRUTURAM AND
                     NREVENTOM = :P_NREVENTOM AND
                     DTINIVIGENCIA = :P_DTINIVIGENCIA AND
                     IDTIPOMOVIMENTO = :P_IDTIPOMOVIMENTO";
                $params = array(
                    "P_NRESTRUTURAM" => $estrutura->getNrestruturam(),
                    "P_NREVENTOM" => $nreventom,
                    "P_DTINIVIGENCIA" => $this->dtinivigencia,
                    "P_IDTIPOMOVIMENTO" => $this->idtipomovimento,
                );
                $eventorelac = FpaEstrutevento::getObjectsByWhere($where, $params);

                if ($eventorelac) {
                    throw new \Exception("Já existe evento ".$nreventom." com mesma vigência e tipo de movimento para a estrutura " . $estrutura->getNrestruturam() ."-". $estrutura->getNmestruturam(), Messages::ERROR_MESSAGE_CODE);
                }
            }
            else {
                throw new \Exception("Estrutura " . $nrestruturam . " não foi encontrada.", Messages::ERROR_MESSAGE_CODE);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrocupacaom
     */
    private function validaOcupacaoEvento($nrocupacaom, $nreventom)
    {
        try {
            /** @var GpeOcupacaom $ocupacao */
            $ocupacao = GpeOcupacaom::getObjectByWhere('NRORG = ? AND NROCUPACAOM = ?', array($this->nrorg, $nrocupacaom));

            if ($ocupacao) {
                $where =
                    "NROCUPACAOM = :P_NROCUPACAOM AND
                     NREVENTOM = :P_NREVENTOM AND
                     DTINIVIGENCIA = :P_DTINIVIGENCIA AND
                     IDTIPOMOVIMENTO = :P_IDTIPOMOVIMENTO";
                $params = array(
                    "P_NROCUPACAOM" => $ocupacao->getNrocupacaom(),
                    "P_NREVENTOM" => $nreventom,
                    "P_DTINIVIGENCIA" => $this->dtinivigencia,
                    "P_IDTIPOMOVIMENTO" => $this->idtipomovimento,
                );
                $eventorelac = FpaOcupaevento::getObjectsByWhere($where, $params);

                if ($eventorelac) {
                    throw new \Exception("Já existe evento ".$nreventom." com mesma vigência e tipo de movimento para a ocupação " . $ocupacao->getNrocupacaom(), Messages::ERROR_MESSAGE_CODE);
                }
            }
            else {
                throw new \Exception("Ocupação " . $nrocupacaom . " não foi encontrada.", Messages::ERROR_MESSAGE_CODE);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaVinculoEvento($nrvinculom, $nreventom)
    {
        try {
            /** @var GpeVinculom $vinculo */
            $vinculo = GpeVinculom::getObjectByWhere('NRORG = ? AND NRVINCULOM = ?', array($this->nrorg, $nrvinculom));

            if ($vinculo) {
                $where =
                    "NRVINCULOM = :P_NRVINCULOM AND
                     NREVENTOM = :P_NREVENTOM AND
                     DTINIVIGENCIA = :P_DTINIVIGENCIA AND
                     IDTIPOMOVIMENTO = :P_IDTIPOMOVIMENTO";
                $params = array(
                    "P_NRVINCULOM" => $vinculo->getNrvinculom(),
                    "P_NREVENTOM" => $nreventom,
                    "P_DTINIVIGENCIA" => $this->dtinivigencia,
                    "P_IDTIPOMOVIMENTO" => $this->idtipomovimento,
                );
                $eventorelac = FpaVincevento::getObjectsByWhere($where, $params);

                if ($eventorelac) {
                    throw new \Exception("Já existe evento ".$nreventom." com mesma vigência e tipo de movimento para o vínculo " . $vinculo->getNrvinculom() ."-". $vinculo->getNmvinculom(), Messages::ERROR_MESSAGE_CODE);
                }
            }
            else {
                throw new \Exception("Vínculo " . $nrvinculom . " não foi encontrado.", Messages::ERROR_MESSAGE_CODE);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereNovoEventoRelacionado($item, $nreventom)
    {
        try {
            if ($this->table == self::ESTRUTURA) {
                $this->validaEstruturaEvento($item, $nreventom);

                $novo = new FpaEstrutevento();
                $novo->setNrestrutevento(NovoCodigo::geraCodigo(FPA::getNrorgTrabalho(), 'FPA_ESTRUTEVENTO'));
                $novo->setNrestruturam($item);
            }
            elseif ($this->table == self::OCUPACAO) {
                $this->validaOcupacaoEvento($item, $nreventom);

                $novo = new FpaOcupaevento();
                $novo->setNrocupaevento(NovoCodigo::geraCodigo(FPA::getNrorgTrabalho(), 'FPA_OCUPAEVENTO'));
                $novo->setNrocupacaom($item);
            }
            elseif ($this->table == self::VINCULO) {
                $this->validaVinculoEvento($item, $nreventom);

                $novo = new FpaVincevento();
                $novo->setNrvincevento(NovoCodigo::geraCodigo(FPA::getNrorgTrabalho(), 'FPA_VINCEVENTO'));
                $novo->setNrvinculom($item);
                $novo->setNrestruturam($this->nrestruturam);
            }

            $novo->setNreventom($nreventom);
            $novo->setDtinivigencia(DateUtil::getDataDeString($this->dtinivigencia,DateUtil::FORMATO_BRASILEIRO,true));
            if ($this->dtfimvigencia)
                $novo->setDtfimvigencia(DateUtil::getDataDeString($this->dtfimvigencia,DateUtil::FORMATO_BRASILEIRO,true));
            $novo->setIdtipomovimento($this->idtipomovimento);
            $novo->setVrpisoevento($this->vrpisoevento);
            $novo->setVrtetoevento($this->vrtetoevento);
            $novo->setNrvalorreferenciam($this->nrvalreferencia);
            $novo->setVrpercent($this->vrpercent);
            $novo->setVrmultiplic($this->vrmultiplic);
            $novo->setVrdivide($this->vrdivide);
            $novo->setVreventofixo($this->vreventofixo);
            $novo->setIdrefgerar($this->idrefgerar);
            $novo->setVrreffixo($this->vrreffixo);
            $novo->setIdretreferencia($this->idretreferencia);
            $novo->setPeriodicidademes($this->periodicidademes);
            $novo->setNrorg($this->nrorg);
            $novo->setCdoperinclusao(FPA::getCdoperador());
            $novo->setDtinclusao(DateUtil::getDataAtual());
            $novo->setNrorginclusao($this->nrorg);
            $novo->setIdgeraafastado($this->idgeraafastado);
            
            $novo->persist();

        } catch (\Exception $e) {
            throw $e;
        }
    }
} 