<?php

require_once 'base/ProdutoEntityBase.php';

/**
 * @author This class has been auto-generated by Setforms.
 *
 */

class ProdutoEntity extends ProdutoEntityBase
{

    const RESULTADO_SEM_REGISTRO = '10';
    const SIM = 'S';
    const NAO = 'N';
    const OUTRO = 'OUTR';
    const ZERO = '0';
    const ZEROS = '00';
    const P = 'P';
    const T = 'T';
    const L = 'L';
    const C = 'C';
    const N = 'N';


    /**
     * Gera novo c?o de produto.
     *
     * @param integer $nrorg N?o da Organiza?.
     *
     * @return integer $codigo
     */
    public static function retornaProxNumeroProduto($nrorg){
        $codigo = Novocodigo::geraCodigo($nrorg, 'PRODUTO', 10);
        return $codigo;
    }


    /**
     * Retorna o registro na tabela de um determinado produto.
     *
     * @param integer $nrorg   N?o da Organiza?.
     * @param integer $produto C?o do Produto.
     *
     * @return ProdutoEntity
     */
    public static function retornaProduto($nrorg, $produto){

        $result = Produto::consultaProduto($nrorg, $produto);

        if ($result instanceof ProdutoEntity) {
            return $result;
        }

        throw new Exception(
            'ProdutoEntity: Nao ha dados sobre este produto!',
            self::RESULTADO_SEM_REGISTRO
        );
    }


    /**
     * Retorna o registro de um determinado produto.
     *
     * @param integer $nrorg     N?o da Organiza?.
     * @param integer $cdarvprod C?o da arvore de Produto.
     *
     * @throws Exception         Sem registro de consulta.
     * @return ProdutoEntity
     */
    public static function retornaArvoreProduto($nrorg, $cdarvprod){
        $result = Produto::consultaArvoreProduto($nrorg, $cdarvprod);
        if ($result instanceof ProdutoEntity) {
            return $result;
        }

        throw new Exception(
            'ProdutoEntity: Nao ha registro de produto para essa consulta.',
            self::RESULTADO_SEM_REGISTRO
        );

    }


     /**
     * Retorna o ultimo item filho de um produto pai.
     *
     * @param integer $nrorg      Numero da organizacao.
     * @param string  $cdarvprod  Codigo de arvore do produto pai.
     *
     * @return Ambigous <<type>, multitype:, mixed>
     */
     public function retornaMaiorItemArvore($nrorg, $cdarvprod){
        $result = Produto::consultaMaiorItemArvore($nrorg, $cdarvprod);
        return $result;
    }


    /**
     * Insere um novo produto.
     *
     * @param integer $cdproduto      Codigo do produto.
     * @param string  $nmproduto      Nome do produto.
     * @param integer $cdnvproduto    Nivel do produto na arvore.
     * @param string  $cdarvprod      Codigo de arvore do produto.
     * @param string  $cdarvpai       Codigo de arvore do item pai.
     * @param integer $nrorg          Numero da organiza?.
     * @param date    $data           Data de inser?.
     * @param integer $operador       Codigo do operador.
     * @param string  $idprodpropterc Idprodproperc.
     *
     * @return boolean
     */
    public function insereProduto(
    $cdproduto, $nmproduto, $cdnvproduto, $cdarvprod, $cdarvpai, $nrorg,
    $data, $operador
    ){

        try {
            $obj = new self();
            $obj->setCdproduto($cdproduto);
            $obj->setNmproduto($nmproduto);
            $obj->setCdnvproduto($cdnvproduto);
            $obj->setCdarvprod($cdarvprod);
            $obj->setCdarvpai($cdarvpai);
            $obj->setNrorg($nrorg);
            $obj->setNrorginclusao($nrorg);
            $obj->setNrorgultatu($nrorg);
            $obj->setDtultatu($data);
            $obj->setDtinclusao($data);
            $obj->setCdoperinclusao($operador);
            $obj->setCdoperultatu($operador);
            $obj->setIdprodpropterc(self::P);
            $obj->setIdcobtxserv(self::SIM);
            $obj->setIdsegmentoprod(self::OUTRO);
            $obj->setIdtipocompprod(self::ZERO);
            $obj->setIdarredtrunc(self::T);
            $obj->setIdsituprod(self::L);
            $obj->setIdprodcontrsubv(self::NAO);
            $obj->setIdunporcaovnd(self::ZERO);
            $obj->setIdpademedcasvnd(self::ZERO);
            $obj->setIdtipomedcasvnd(self::ZEROS);
            $obj->setIdorigprodpr(self::T);
            $obj->setIdtiporetiore(self::C);
            $obj->setIdretisemestore(self::N);

            $obj->persist();
            return true;

        } catch (Exception $e) {
            tk_warning('Erro desconhecido '. $e->getMessage());
            return false;
        }
    }


    /**
     * Atualiza produto.
     *
     * @param integer $cdproduto Codigo do produto.
     * @param string  $cdarvprod Codigo de arvore do produto.
     * @param string  $nmproduto Nome do produto.
     * @param integer $nrorg     N?o da organiza? de atualiza?.
     * @param date    $data      Data de atualiza?.
     * @param integer $operador  Operador de atualiza?.
     *
     * @return boolean
     */
    public function atualizaProduto(
    $cdproduto, $cdarvprod, $nmproduto, $nrorg, $data, $operador
    ){

        $lista_registro = Produto::getObjectsByWhere(
            'NRORG = ? AND CDPRODUTO = ?',
            array(
             $nrorg,
             $cdproduto,
            )
        );

        foreach ($lista_registro as $produto) {
            $produto->setCdarvprod($cdarvprod);
            $produto->setNmproduto($nmproduto);
            $produto->setNrorgultatu($nrorg);
            $produto->setDtultatu($data);
            $produto->setCdoperultatu($operador);

            $produto->persist();
        }

        return true;
    }


    public static function produtoCustoLocalEstoque(
        $cdfilial,
        $cdproduto,
        $nrtipocusto,
        $dtreferencia,
        $dtatual,
        $almoxarife,
        $localestoq,
        $nrloteestq,
        $nrsublote
    ) {
        $result = Produto::produtoCustoLocalEstoque(
            $cdfilial,
            $cdproduto,
            $nrtipocusto,
            $dtreferencia,
            $dtatual,
            $almoxarife,
            $localestoq,
            $nrloteestq,
            $nrsublote
        );

        return $result;
    }


    public static function excluiFilhosItemProduto($nrorg, $cdproduto){
        Produto::excluiFilhosItemProduto($nrorg, $cdproduto);
    }


    public static function consultaPeriodicidadeProduto($cdfilsoli, $cdproduto, $nrorg){
        return Produto::consultaPeriodicidadeProduto($cdfilsoli, $cdproduto, $nrorg);
    }


    public static function retornaFilhosProduto($nrorg, $cdarvprod){
        $result = Produto::consultaFilhosProduto($nrorg, $cdarvprod);
        if (is_array($result) === true) {
            return $result;
        }

        throw new Exception(
            'ProdutoEntity: Nao ha registro de produto para essa consulta.',
            self::RESULTADO_SEM_REGISTRO
        );

    }


}//end class

